@@ -30,6 +30,7 @@
 
 static t_cached_object *cache[MAX_CACHE_INDEX];
 static pthread_mutex_t cache_mutex[MAX_CACHE_INDEX];
+static pthread_mutex_t cachesize_mutex;
 static volatile off_t cachesize;
 
 void init_cache_module(void) {
@@ -40,6 +41,7 @@ void init_cache_module(void) {
 		pthread_mutex_init(&cache_mutex[i], NULL);
 	}
 
+	pthread_mutex_init(&cachesize_mutex, NULL);
 	cachesize = 0;
 }
 
@@ -97,7 +99,10 @@ static t_cached_object *remove_from_cache(t_cached_object *object, int index) {
 		object->next->prev = object->prev;
 	}
 
+	pthread_mutex_lock(&cachesize_mutex);
 	cachesize -= object->size;
+	pthread_mutex_unlock(&cachesize_mutex);
+
 	if (object == cache[index]) {
 		cache[index] = object->next;
 	}
@@ -167,7 +172,13 @@ t_cached_object *add_to_cache(t_session *session, char *file) {
 	object->in_use = 1;
 	copy_ip(&(object->last_ip), &(session->ip_address));
 
-	i = cache_index(file);
+	if ((i = cache_index(file)) == -1) {
+		free(object->data);
+		free(object->file);
+		free(object);
+
+		return NULL;
+	}
 
 	pthread_mutex_lock(&cache_mutex[i]);
 
@@ -178,7 +189,9 @@ t_cached_object *add_to_cache(t_session *session, char *file) {
 	}
 	cache[i] = object;
 
+	pthread_mutex_lock(&cachesize_mutex);
 	cachesize += object->size;
+	pthread_mutex_unlock(&cachesize_mutex);
 
 	pthread_mutex_unlock(&cache_mutex[i]);
 
@@ -197,7 +210,9 @@ t_cached_object *search_cache(t_session *session, char *file) {
 		return NULL;
 	}
 
-	i = cache_index(file);
+	if ((i = cache_index(file)) == -1) {
+		return NULL;
+	}
 
 	pthread_mutex_lock(&cache_mutex[i]);
 
@@ -279,7 +294,9 @@ int clear_cache(void) {
 			if (object->in_use == 0) {
 				/* Object unused, so remove
 				 */
+				pthread_mutex_lock(&cachesize_mutex);
 				cachesize -= object->size;
+				pthread_mutex_unlock(&cachesize_mutex);
 				
 				free(object->data);
 				free(object->file);
