@@ -23,7 +23,7 @@ typedef struct type_cached_object {
 	char          *file;
 	char          *data;
 	off_t         size;
-	int           timer;
+	time_t        deadline;
 	time_t        last_changed;
 	volatile int  in_use;
 	t_ip_addr     last_ip;
@@ -36,7 +36,7 @@ void init_cache_module(void);
 t_cached_object *add_to_cache(t_session *session, char *file);
 t_cached_object *search_cache(t_session *session, char *file);
 void done_with_cached_object(t_cached_object *object, bool remove_object);
-void check_cache(void);
+void check_cache(time_t time);
 int clear_cache(void);
 #ifdef HAVE_COMMAND
 void print_cache_list(FILE *fp);
