@@ -19,6 +19,9 @@
 #include <pthread.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -512,7 +515,7 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
  *  ================
  */
 int connect_to_fcgi_server(t_connect_to *connect_to) {
-	int sock = -1;
+	int sock = -1, max_len;
 	struct sockaddr_un sunix;
 	struct sockaddr_in saddr4;
 #ifdef HAVE_IPV6
@@ -528,7 +531,9 @@ int connect_to_fcgi_server(t_connect_to *connect_to) {
 		 */
 		if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) > 0) {
 			sunix.sun_family = AF_UNIX;
-			strncpy(sunix.sun_path, connect_to->unix_socket, (sizeof(struct sockaddr_un) - sizeof(short)));
+			max_len = sizeof(sunix.sun_path);
+			strncpy(sunix.sun_path, connect_to->unix_socket, max_len);
+			sunix.sun_path[max_len - 1] = '\0';
 			if (connect(sock, (struct sockaddr*)&sunix, sizeof(struct sockaddr_un)) != 0) {
 				close(sock);
 				sock = -1;
