@@ -522,7 +522,7 @@ int register_wrong_password(t_session *session) {
 	while (item != NULL) {
 		if (same_ip(&(item->ip), &(session->ip_address))) {
 			if (++(item->count) >= session->config->max_wrong_passwords) {
-				if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				if (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny) {
 					ban_ip(&(session->ip_address), session->config->ban_on_wrong_password, session->config->kick_on_ban);
 					session->keep_alive = false;
 					log_system(session, "Client banned because of too many wrong passwords");
