@@ -85,6 +85,9 @@
 
 #define RANDOM_BUFFER_SIZE     512
 
+#define NO_REQUEST_LIMIT_TIME  300
+#define NO_REQUEST_LIMIT_SIZE    8 * MEGABYTE
+
 typedef struct {
 	char *config_dir;
 	bool daemon;
@@ -192,26 +195,18 @@ t_cgi_type check_target_is_cgi(t_session *session) {
 /* Handle an HTTP error.
  */
 int handle_error(t_session *session, int error_code) {
-	char *error_handler = NULL, *new_fod;
+	t_error_handler *error_handler;
+	char *new_fod;
 	int result = -1;
 
-	switch (error_code) {
-		case 401:
-			error_handler = session->host->error_401_handler;
-			break;
-		case 403:
-			error_handler = session->host->error_403_handler;
-			break;
-		case 404:
-			error_handler = session->host->error_404_handler;
-			break;
-		case 501:
-			error_handler = session->host->error_501_handler;
-			break;
-		case 503:
-			error_handler = session->host->error_503_handler;
+	error_handler = session->host->error_handlers;
+	while (error_handler != NULL) {
+		if (error_handler->code == error_code) {
 			break;
+		}
+		error_handler = error_handler->next;
 	}
+
 	if (error_handler == NULL) {
 		return 0;
 	}
@@ -220,7 +215,9 @@ int handle_error(t_session *session, int error_code) {
 	session->error_code = error_code;
 	session->handling_error = true;
 	session->mimetype = NULL;
-	if ((new_fod = (char*)malloc(session->host->website_root_len + strlen(error_handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
+	session->vars = error_handler->parameters;
+
+	if ((new_fod = (char*)malloc(session->host->website_root_len + strlen(error_handler->handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
 		result = 500;
 	} else {
 		if (session->file_on_disk != NULL) {
@@ -229,7 +226,7 @@ int handle_error(t_session *session, int error_code) {
 		session->file_on_disk = new_fod;
 
 		memcpy(session->file_on_disk, session->host->website_root, session->host->website_root_len);
-		strcpy(session->file_on_disk + session->host->website_root_len, error_handler);
+		strcpy(session->file_on_disk + session->host->website_root_len, error_handler->handler);
 
 		if (get_target_extension(session) == -1) {
 			return 500;
@@ -272,11 +269,11 @@ int handle_error(t_session *session, int error_code) {
 			log_error(session, "ErrorHandler not found");
 			break;
 		case 500:
-			log_file_error(session, error_handler, "internal error for ErrorHandler");
+			log_file_error(session, error_handler->handler, "internal error for ErrorHandler");
 			session->keep_alive = false;
 			break;
 		case 503:
-			log_file_error(session, error_handler, "FastCGI for ErrorHandler not available");
+			log_file_error(session, error_handler->handler, "FastCGI for ErrorHandler not available");
 			break;
 	}
 
@@ -356,18 +353,25 @@ int run_program(t_session *session, char *program, int return_code) {
  */
 int fetch_request(t_session *session) {
 	char *new_reqbuf, *strstart, *strend;
-	long bytes_read, header_length = -1, content_length = -1;
+	long max_request_size, bytes_read, header_length = -1, content_length = -1;
 	int result = 200, write_bytes;
 	time_t deadline;
 	fd_set read_fds;
 	struct timeval select_timeout;
-	bool keep_reading = true, store_on_disk = false;
+	bool keep_reading = true, store_on_disk = false, request_limit;
 	int upload_handle = -1;
 
-	if (session->kept_alive == 0) {
+	request_limit = ip_allowed(&(session->ip_address), session->config->request_limit_mask) != deny;
+
+	if (request_limit == false) {
+		deadline = session->time + NO_REQUEST_LIMIT_TIME;
+		max_request_size = NO_REQUEST_LIMIT_SIZE;
+	} else if (session->kept_alive == 0) {
 		deadline = session->time + session->binding->time_for_1st_request;
+		max_request_size = session->binding->max_request_size;
 	} else {
 		deadline = session->time + session->binding->time_for_request;
+		max_request_size = session->binding->max_request_size;
 	}
 
 	do {
@@ -437,7 +441,7 @@ int fetch_request(t_session *session) {
 								}
 							} else {
 								session->content_length = content_length;
-								if (header_length + content_length > session->binding->max_request_size) {
+								if (header_length + content_length > max_request_size) {
 									result = fe_MAX_REQUESTSIZE;
 									break;
 								}
@@ -558,7 +562,7 @@ int fetch_request(t_session *session) {
 							session->bytes_in_buffer += bytes_read;
 							*(session->request + session->bytes_in_buffer) = '\0';
 
-							if (session->bytes_in_buffer > session->binding->max_request_size) {
+							if (session->bytes_in_buffer > max_request_size) {
 								keep_reading = false;
 								result = fe_MAX_REQUESTSIZE;
 								break;
@@ -704,11 +708,7 @@ int uri_to_path(t_session *session) {
 				if ((retval = get_homedir(session, session->local_user + 1)) != 200) {
 					return retval;
 				}
-				session->host->error_401_handler = NULL;
-				session->host->error_403_handler = NULL;
-				session->host->error_404_handler = NULL;
-				session->host->error_501_handler = NULL;
-				session->host->error_503_handler = NULL;
+				session->host->error_handlers = NULL;
 			} else {
 				/* uri is '/~/...' */
 				return 404;
@@ -908,7 +908,7 @@ int serve_client(t_session *session) {
 		deny_body = session->host->deny_body;
 		while (deny_body != NULL) {
 			if (strpcmp(session->body, &(deny_body->pattern)) == 0) {
-				if ((session->config->ban_on_denied_body > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+				if ((session->config->ban_on_denied_body > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 					ban_ip(&(session->ip_address), session->config->ban_on_denied_body, session->config->kick_on_ban);
 					log_system(session, "Client banned because of denied body");
 #ifdef HAVE_MONITOR
@@ -949,7 +949,7 @@ int serve_client(t_session *session) {
 				return 500;
 			}
 
-			if ((options.ban > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+			if ((options.ban > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				ban_ip(&(session->ip_address), options.ban, session->config->kick_on_ban);
 				log_system(session, "Client banned because of URL match in UrlToolkit rule");
 #ifdef HAVE_MONITOR
@@ -1175,6 +1175,22 @@ int serve_client(t_session *session) {
 	return result;
 }
 
+/* Handle timeout upon sending request
+ */
+void handle_timeout(t_session *session) {
+	if ((session->config->ban_on_timeout > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
+		ban_ip(&(session->ip_address), session->config->ban_on_timeout, session->config->kick_on_ban);
+		log_system(session, "Client banned because of connection timeout");
+#ifdef HAVE_MONITOR
+		if (session->config->monitor_enabled) {
+			monitor_counter_ban(session);
+		}
+#endif
+	} else {
+		log_system(session, "Timeout while waiting for request");
+	}
+}
+
 /* Request has been handled, handle the return code.
  */
 void handle_request_result(t_session *session, int result) {
@@ -1183,7 +1199,7 @@ void handle_request_result(t_session *session, int result) {
 			log_system(session, "Maximum request size reached");
 			session->return_code = 413;
 			send_code(session);
-			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+			if ((session->config->ban_on_max_request_size > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_max_request_size, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending a too large request");
 #ifdef HAVE_MONITOR
@@ -1197,17 +1213,7 @@ void handle_request_result(t_session *session, int result) {
 			if (session->kept_alive == 0) {
 				session->return_code = 408;
 				send_code(session);
-				if ((session->config->ban_on_timeout > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
-					ban_ip(&(session->ip_address), session->config->ban_on_timeout, session->config->kick_on_ban);
-					log_system(session, "Client banned because of connection timeout");
-#ifdef HAVE_MONITOR
-					if (session->config->monitor_enabled) {
-						monitor_counter_ban(session);
-					}
-#endif
-				} else {
-					log_system(session, "Timeout while waiting for request");
-				}
+				handle_timeout(session);
 			}
 			break;
 		case fe_CLIENT_DISCONNECTED:
@@ -1224,7 +1230,7 @@ void handle_request_result(t_session *session, int result) {
 			log_system(session, "Client kicked");
 			break;
 		case rr_SQL_INJECTION:
-			if ((session->config->ban_on_sqli > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+			if ((session->config->ban_on_sqli > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_sqli, session->config->kick_on_ban);
 				log_system(session, "Client banned because of SQL injection");
 #ifdef HAVE_MONITOR
@@ -1266,7 +1272,7 @@ void handle_request_result(t_session *session, int result) {
 					session->keep_alive = false;
 				}
 			}
-			if ((session->config->ban_on_garbage > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+			if ((session->config->ban_on_garbage > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_garbage, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending garbage");
 #ifdef HAVE_MONITOR
@@ -1317,7 +1323,7 @@ void handle_request_result(t_session *session, int result) {
 /* Handle the connection of a client.
  */
 void connection_handler(t_session *session) {
-	int result, connections;
+	int result, connections, timeout;
 
 #ifdef HAVE_MONITOR
 	connections = ++open_connections;
@@ -1330,7 +1336,15 @@ void connection_handler(t_session *session) {
 
 #ifdef HAVE_SSL
 	if (session->binding->use_ssl) {
-		session->socket_open = (ssl_accept(session->client_socket, &(session->ssl_data), session->binding->ssl_context) != -1);
+		timeout = session->kept_alive == 0 ? session->binding->time_for_1st_request : session->binding->time_for_request;
+		switch (ssl_accept(session->client_socket, &(session->ssl_data), session->binding->ssl_context, timeout)) {
+			case -2:
+				handle_timeout(session);
+				break;
+			case 0:
+				session->socket_open = true;
+				break;
+		}
 	} else
 #endif
 		session->socket_open = true;
@@ -1356,7 +1370,7 @@ void connection_handler(t_session *session) {
 
 			if (session->keep_alive && (session->config->ban_on_flooding > 0)) {
 				if (client_is_flooding(session)) {
-					if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+					if (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny) {
 						ban_ip(&(session->ip_address), session->config->ban_on_flooding, session->config->kick_on_ban);
 						log_system(session, "Client banned because of flooding");
 						session->keep_alive = false;
@@ -1683,6 +1697,7 @@ int accept_connection(t_binding *binding, t_config *config) {
 #endif
 	} else {
 		log_system(session, "Incoming connection via unknown protocol");
+		free(session);
 		return -1;
 	}
 
@@ -1734,7 +1749,7 @@ int accept_connection(t_binding *binding, t_config *config) {
 	} else switch (total_conns) {
 		case ca_TOOMUCH_PERIP:
 			log_system(session, "Maximum number of connections for IP address reached");
-			if ((config->ban_on_max_per_ip > 0) && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+			if ((config->ban_on_max_per_ip > 0) && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				log_system(session, "Client banned because of too many simultaneous connections");
 				ban_ip(&(session->ip_address), config->ban_on_max_per_ip, config->kick_on_ban);
 #ifdef HAVE_MONITOR
@@ -1748,7 +1763,7 @@ int accept_connection(t_binding *binding, t_config *config) {
 			log_system(session, "Maximum number of total connections reached");
 			break;
 		case ca_BANNED:
-			if (config->reban_during_ban && ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
+			if (config->reban_during_ban && (ip_allowed(&(session->ip_address), session->config->banlist_mask) != deny)) {
 				reban_ip(&(session->ip_address));
 			}
 #ifdef HAVE_COMMAND
@@ -2119,7 +2134,7 @@ int run_server(t_settings *settings) {
 			case -1:
 				if (errno != EINTR) {
 					log_string(config->system_logfile, "Fatal error selecting connection");
-					sleep(1);
+					usleep(1000);
 				}
 				break;
 			case 0:
