@@ -88,13 +88,11 @@ bool group_oke(t_session *session, char *user, t_charlist *group) {
 	return retval;
 }
 
-/* Read a password from a file
- */
-static char *get_password(t_session *session, char *username) {
-	char line[257], *result = NULL, *sep;
-	FILE *pfp;
+static FILE *open_password_file(t_session *session) {
+	char *result;
+	FILE *fp;
 	
-	if ((pfp = fopen_neighbour(session->host->passwordfile, "r", session->file_on_disk)) == NULL) {
+	if ((fp = fopen_neighbour(session->host->passwordfile, "r", session->file_on_disk)) == NULL) {
 		switch (errno) {
 			case EACCES:
 				result = "no access to password file";
@@ -109,30 +107,90 @@ static char *get_password(t_session *session, char *username) {
 		return NULL;
 	}
 
+	return fp;
+}
+
+/* Get password from password file
+ */
+static char *get_password(t_session *session, char *username) {
+	char line[257], *result = NULL, *sep;
+	FILE *fp;
+
+	if ((fp = open_password_file(session)) == NULL) {
+		return NULL;
+	}
+
 	line[256] = '\0';
-	while ((sep = fgets(line, 256, pfp)) != NULL) {
-		while ((*sep != ':') && (*sep != '\0')) {
+	while ((sep = fgets(line, 256, fp)) != NULL) {
+		if ((sep = strchr(sep, ':')) == NULL) {
+			continue;
+		}
+
+		*(sep++) = '\0';
+		if (strcmp(line, username) != 0) {
+			continue;
+		}
+
+		result = sep;
+		while ((*sep != '\n') && (*sep != '\r') && (*sep != ':') && (*sep != '\0')) {
 			sep++;
 		}
-		if (*sep == ':') {
-			*(sep++) = '\0';
-			if (strcmp(line, username) == 0) {
-				result = sep;
-
-				while ((*sep != '\n') && (*sep != '\r') && (*sep != ':') && (*sep != '\0')) {
-					sep++;
-				}
-				if (*sep != '\0') {
-					*sep = '\0';
-					result = strdup(result);
-				} else {
-					result = NULL;
-				}
-				break;
-			}
+		if (*sep != '\0') {
+			*sep = '\0';
+			result = strdup(result);
+		} else {
+			result = NULL;
 		}
+		break;
 	}
-	fclose(pfp);
+	fclose(fp);
+
+	return result;
+}
+
+/* Get password (A1) from password file
+ */
+static char *get_A1(t_session *session, char *username, char *realm) {
+	char line[257], *result = NULL, *sep1, *sep2;
+	FILE *fp;
+
+	if ((fp = open_password_file(session)) == NULL) {
+		return NULL;
+	}
+
+	line[256] = '\0';
+	while ((sep1 = fgets(line, 256, fp)) != NULL) {
+		if ((sep1 = strchr(sep1, ':')) == NULL) {
+			continue;
+		}
+
+		*(sep1++) = '\0';
+		if (strcmp(line, username) != 0) {
+			continue;
+		}
+
+		if ((sep2 = strchr(sep1, ':')) == NULL) {
+			continue;
+		}
+
+		*(sep2++) = '\0';
+		if (strcmp(sep1, realm) != 0) {
+			continue;
+		}
+
+		result = sep2;
+		while ((*sep2 != '\n') && (*sep2 != '\r') && (*sep2 != ':') && (*sep2 != '\0')) {
+			sep2++;
+		}
+		if (*sep2 != '\0') {
+			*sep2 = '\0';
+			result = strdup(result);
+		} else {
+			result = NULL;
+		}
+		break;
+	}
+	fclose(fp);
 
 	return result;
 }
@@ -201,7 +259,7 @@ static char *unquoted(char *str) {
 /* Digest HTTP authentication.
  */
 static bool digest_http_authentication(t_session *session, char *auth_str) {
-	bool retval = false, quote_found;
+	bool quote_found;
 	char *key, *value, *rest, *empty = "", *passwd, A1[33], A2[33], result[33];
 	char *username = empty, *realm = empty, *nonce = empty, *uri = empty, *qop = empty,
 		 *nc = empty, *cnonce = empty, *algoritm = empty, *response = empty, *opaque = empty;
@@ -261,47 +319,49 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 		return false;
 	}
 
-	if ((passwd = get_password(session, username)) != NULL) {
-		if (group_oke(session, username, &(session->host->required_group))) do {
-			/* Calculate A1
-			 */
-			if ((value = (char*)malloc(strlen(username) + strlen(realm) + strlen(passwd) + 3)) == NULL) {
-				break;
-			}
-			sprintf(value, "%s:%s:%s", username, realm, passwd);
-			md5_hash(value, strlen(value), A1);
-			free(value);
-
-			/* Calculate A2
-			 */
-			if ((value = (char*)malloc(strlen(session->method) + strlen(uri) + 2)) == NULL) {
-				break;
-			}
-			sprintf(value, "%s:%s", session->method, uri);
-			md5_hash(value, strlen(value), A2);
-			free(value);
-
-			/* Calculate response
-			 */
-			if ((value = (char*)malloc(strlen(A1) + strlen(nonce) + strlen(A2) + 6)) == NULL) {
-				break;
-			}
-			sprintf(value, "%s:%s:%s", A1, nonce, A2);
-			md5_hash(value, strlen(value), result);
-			free(value);
-
-			/* Password match?
-			 */
-			if (strcmp(result, response) == 0) {
-				retval = ((session->remote_user = strdup(username)) != NULL);
-			} else {
-				register_wrong_password(session);
-			}
-		} while (false);
+	/* Retrieve A1 from passwordfile
+	 */
+	if ((passwd = get_A1(session, username, realm)) == NULL) {
+		return false;
+	} else if (strlen(passwd) != 32) {
 		free(passwd);
+		return false;
+	}
+	memcpy(A1, passwd, 33);
+	free(passwd);
+
+	/* Group oke?
+	 */
+	if (group_oke(session, username, &(session->host->required_group)) == false) {
+		return false;
+	}
+	
+	/* Calculate A2
+	 */
+	if ((value = (char*)malloc(strlen(session->method) + strlen(uri) + 2)) == NULL) {
+		return false;
+	}
+	sprintf(value, "%s:%s", session->method, uri);
+	md5_hash(value, strlen(value), A2);
+	free(value);
+
+	/* Calculate response
+	 */
+	if ((value = (char*)malloc(strlen(A1) + strlen(nonce) + strlen(A2) + 6)) == NULL) {
+		return false;
+	}
+	sprintf(value, "%s:%s:%s", A1, nonce, A2);
+	md5_hash(value, strlen(value), result);
+	free(value);
+
+	/* Password match?
+	 */
+	if (strcmp(result, response) != 0) {
+		register_wrong_password(session);
+		return false;
 	}
 
-	return retval;
+	return (session->remote_user = strdup(username)) != NULL;
 }
 
 /* Check if the file is protected by an .hiawatha file with passwordfile setting.
