@@ -435,6 +435,65 @@ t_keyvalue *remove_keyvaluelist(t_keyvalue *list) {
 	return NULL;
 }
 
+/*---< error handlers >------------------------------------------------------*/
+
+int parse_error_handler(char *line, t_error_handler **handlers) {
+	t_error_handler *handler;
+	char *param;
+	int code;
+
+	if (line == NULL) {
+		return -1;
+	} else if ((handler = (t_error_handler*)malloc(sizeof(t_error_handler))) == NULL) {
+		return -1;
+	}
+
+	if (split_string(line, &param, &line, ':') != 0) {
+		free(handler);
+		return -1;
+	}
+
+	switch (code = str2int(param)) {
+		case 401:
+		case 403:
+		case 404:
+		case 501:
+		case 503:
+			handler->code = code;
+			break;
+		default:
+			free(handler);
+			return -1;
+	}
+
+	if ((strlen(line) > 128) || (*line != '/')) {
+		free(handler);
+		return -1;
+	} else if ((handler->handler = strdup(line)) == NULL) {
+		free(handler);
+		return -1;
+	}
+
+	if ((param = strchr(handler->handler, '?')) != NULL) {
+		*param = '\0';
+		handler->parameters = param + 1;
+	} else {
+		handler->parameters = NULL;
+	}
+
+	handler->next = *handlers;
+	*handlers = handler;
+
+	return 0;
+}
+
+void remove_error_handler(t_error_handler *handler) {
+	if (handler != NULL) {
+		free(handler->handler);
+		free(handler);
+	}
+}
+
 /*---< temporary data >------------------------------------------------------*/
 
 int register_tempdata(t_tempdata **tempdata, void *data, t_tempdata_type type) {
@@ -478,6 +537,9 @@ void remove_tempdata(t_tempdata *tempdata) {
 			case tc_charlist:
 				remove_charlist((t_charlist*)tdata->content);
 				break;
+			case tc_errorhandler:
+				remove_error_handler((t_error_handler*)tdata->content);
+				break;
 		}
 		free(tdata);
 	}
