@@ -1,3 +1,14 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License. For a copy,
+ * see http://www.gnu.org/licenses/gpl-2.0.html.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ */
+
 #include "config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -105,6 +116,20 @@ char *strlower(char *str) {
 	return str;
 }
 
+/* Convert a hexadecimal char to an integer.
+ */
+short hex_to_int(char c) {
+	 if ((c >= '0') && (c <= '9')) {
+		 return c - '0';
+	 } else if ((c >= 'A') && (c <= 'F')) {
+		 return c - 'A' + 10;
+	 } else if ((c >= 'a') && (c <= 'f')) {
+		 return c - 'a' + 10;
+	 }
+
+	 return -1;
+}
+
 /* Split a string in 2 strings.
  */
 int split_string(const char *str, char **key, char **value, char c) {
@@ -193,20 +218,6 @@ bool valid_uri(char *uri) {
 	return true;
 }
 
-/* Convert a hexadecimal char to an integer.
- */
-static short hex_to_int(char c) {
-	 if ((c >= '0') && (c <= '9')) {
-		 return c - '0';
-	 } else if ((c >= 'A') && (c <= 'F')) {
-		 return c - 'A' + 10;
-	 } else if ((c >= 'a') && (c <= 'f')) {
-		 return c - 'a' + 10;
-	 }
-
-	 return -1;
-}
-
 /* Encode a string to an URL encoded one
  */
 static bool char_needs_encoding(char c) {
@@ -305,89 +316,6 @@ bool forbidden_chars_present(char *str) {
 	return false;
 }
 
-/* Prevent SQL injection
- */
-int prevent_sqli(char *str, int length, char **newstr) {
-	int changed = 0, i, add, left, newlength = 0;
-	char *temp, value;
-	short low, high;
-
-	if ((str == NULL) || (length == 0) || (newstr == NULL)) {
-		return 0;
-	}
-
-	add = left = 32;
-	if ((*newstr = (char*)malloc(length + add + 1)) == NULL) {
-		return -1;
-	}
-	for (i = 0; i < length; i++) {
-		value = *(str + i);
-
-		if ((value == '%') && (i + 2 < length)) {
-			if ((high = hex_to_int(*(str + i + 1))) != -1) {
-				if ((low = hex_to_int(*(str + i + 2))) != -1) {
-					value = (char)(high<<4) + low;
-				}
-			}
-		}
-
-		if ((value == '\'') || (value == '`') || (value == '\\')) {
-			*(*newstr + newlength++) = '\\';
-			if (left-- == 0) {
-				add += left = 32;
-				if ((temp = (char*)realloc(*newstr, length + add + 1)) == NULL) {
-					free(*newstr);
-					return -1;
-				} else {
-					*newstr = temp;
-				}
-			}
-			changed++;
-		}
-		*(*newstr + (newlength++)) = *(str + i);
-	}
-
-	if (changed == 0) {
-		free(*newstr);
-		*newstr = NULL;
-		newlength = 0;
-	} else {
-		*(*newstr + newlength) = '\0';
-	}
-
-	return newlength;
-}
-
-/* Prevent cross-site scripting.
- */
-int prevent_xss(char *str) {
-	int result = 0;
-	short low, high;
-	char value;
-
-	if (str == NULL) {
-		return 0;
-	}
-
-	while (*str != '\0') {
-		if ((value = *str) == '%') {
-			if ((high = hex_to_int(*(str + 1))) != -1) {
-				if ((low = hex_to_int(*(str + 2))) != -1) {
-					value = (char)(high<<4) + low;
-				}
-			}
-		}
-
-		if ((value == '\"') || (value == '<') || (value == '>') || (value == '\'')) {
-			*str = '_';
-			result += 1;
-		}
-		str++;
-	}
-
-	return result;
-}
-
 /* Return an errormessage.
  */
 const char *http_error(int code) {
