@@ -416,7 +416,11 @@ int send_code(t_session *session) {
 				return -1;
 			}
 
-			if (send_buffer(session, *(session->host->hostname.item), strlen(*(session->host->hostname.item))) == -1) {
+			if (session->hostname != NULL) {
+				if (send_buffer(session, session->hostname, strlen(session->hostname)) == -1) {
+					return -1;
+				}
+			} else if (send_buffer(session, *(session->host->hostname.item), strlen(*(session->host->hostname.item))) == -1) {
 				return -1;
 			}
 
