@@ -54,11 +54,7 @@ static t_host *new_host(void) {
 	host->website_root       = NULL;
 	host->website_root_len   = 0;
 	host->start_file         = "index.html";
-	host->error_401_handler  = NULL;
-	host->error_403_handler  = NULL;
-	host->error_404_handler  = NULL;
-	host->error_501_handler  = NULL;
-	host->error_503_handler  = NULL;
+	host->error_handlers     = NULL;
 	host->access_logfile     = LOG_DIR"/access.log";
 	host->access_fileptr     = NULL;
 	host->access_fp          = &(host->access_fileptr);
@@ -273,6 +269,7 @@ t_config *default_config(void) {
 	config->user_directory     = "public_html";
 	config->user_directory_set = false;
 	config->hide_proxy         = NULL;
+	config->request_limit_mask = NULL;
 
 	config->pidfile            = PIDFILE_DIR"/hiawatha.pid";
 	config->system_logfile     = LOG_DIR"/system.log";
@@ -586,6 +583,11 @@ int check_configuration(t_config *config) {
 			return -1;
 		}
 
+		if ((host->wrap_cgi != NULL) && (config->cgi_wrapper == NULL)) {	
+			fprintf(stderr, "WrapCGI set for host '%s' while CGIwrapper is unset.\n", *(host->hostname.item));
+			return -1;
+		}
+
 		host = host->next;
 	}
 
@@ -920,6 +922,10 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if ((config->reconnect_delay = str2int(value)) > 0) {
 			return true;
 		}
+	} else if (strcmp(key, "requestlimitmask") == 0) {
+		if ((config->request_limit_mask = parse_accesslist(value, false, config->request_limit_mask)) != NULL) {
+			return true;
+		}
 	} else if (strcmp(key, "serverid") == 0) {
 		split_string(value, &uid, &gid, ':');
 		if (parse_userid(uid, &(config->server_uid)) == 1) {
@@ -1013,7 +1019,8 @@ static bool system_setting(char *key, char *value, t_config *config) {
 }
 
 static bool user_setting(char *key, char *value, t_host *host, t_tempdata **tempdata) {
-	char *code, **handler, *pwd = NULL, *grp = NULL;
+	char *pwd = NULL, *grp = NULL;
+	t_error_handler *handler;
 	t_keyvalue *kv;
 
 	if (strcmp(key, "accesslist") == 0) {
@@ -1033,37 +1040,18 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 			return true;
 		}
 	} else if (strcmp(key, "errorhandler") == 0) {
-		if (split_string(value, &code, &value, ':') == 0) {
-			switch (str2int(code)) {
-				case 401:
-					handler = &(host->error_401_handler);
-					break;
-				case 403:
-					handler = &(host->error_403_handler);
-					break;
-				case 404:
-					handler = &(host->error_404_handler);
-					break;
-				case 501:
-					handler = &(host->error_501_handler);
-					break;
-				case 503:
-					handler = &(host->error_503_handler);
-					break;
-				default:
-					return false;
-			}
-			if ((strlen(value) < 128) && (*value == '/')) {
-				if ((*handler = strdup(value)) != NULL) {
-					if (register_tempdata(tempdata, *handler, tc_data) == -1) {
-						free(*handler);
-						*handler = NULL;
-						return false;
-					}
-					return true;
-				}
-			}
+		if (parse_error_handler(value, &(host->error_handlers)) == -1) {
+			return false;
 		}
+
+		if (register_tempdata(tempdata, host->error_handlers, tc_errorhandler) == -1) {
+			handler = host->error_handlers;
+			host->error_handlers = host->error_handlers->next;
+			remove_error_handler(handler);
+			return false;
+		}
+
+		return true;
 	} else if (strcmp(key, "loginmessage") == 0) {
 		if (strlen(value) < 64) {
 			if ((host->login_message = strdup(value)) != NULL) {
