@@ -170,73 +170,68 @@ typedef struct type_directory {
 } t_directory;
 
 typedef struct type_host {
-	char          *website_root;
-	size_t        website_root_len;
-	char          *start_file;
-	char          *error_401_handler;
-	char          *error_403_handler;
-	char          *error_404_handler;
-	char          *error_501_handler;
-	char          *error_503_handler;
-	char          *access_logfile;
-	FILE          *access_fileptr;
-	FILE          **access_fp;
-	time_t        access_time;
-	char          *error_logfile;
-	t_charlist    hostname;
-	bool          user_websites;
-	bool          execute_cgi;
-	int           time_for_cgi;
+	char            *website_root;
+	size_t          website_root_len;
+	char            *start_file;
+	t_error_handler *error_handlers;
+	char            *access_logfile;
+	FILE            *access_fileptr;
+	FILE            **access_fp;
+	time_t          access_time;
+	char            *error_logfile;
+	t_charlist      hostname;
+	bool            user_websites;
+	bool            execute_cgi;
+	int             time_for_cgi;
 #ifdef HAVE_XSLT
-	char          *show_index;
+	char            *show_index;
 #endif
-	bool          use_gz_file;
-	char          *login_message;
-	char          *passwordfile;
-	t_auth_method auth_method;
-	char          *groupfile;
-	t_charlist    required_binding;
-	t_denybotlist *deny_bot;
-	t_charlist    required_group;
-	t_charlist    alter_group;
-	char          *wrap_cgi;
-	t_groups      groups;
-	t_charlist    volatile_object;
-	t_accesslist  *access_list;
-	t_accesslist  *alter_list;
-	mode_t        alter_fmode;
-	char          *run_on_alter;
-	t_charlist    image_referer;
-	char          *imgref_replacement;
-	t_keyvalue    *envir_str;
-	t_keyvalue    *alias;
+	bool            use_gz_file;
+	char            *login_message;
+	char            *passwordfile;
+	t_auth_method   auth_method;
+	char            *groupfile;
+	t_charlist      required_binding;
+	t_denybotlist   *deny_bot;
+	t_charlist      required_group;
+	t_charlist      alter_group;
+	char            *wrap_cgi;
+	t_groups        groups;
+	t_charlist      volatile_object;
+	t_accesslist    *access_list;
+	t_accesslist    *alter_list;
+	mode_t          alter_fmode;
+	char            *run_on_alter;
+	t_charlist      image_referer;
+	char            *imgref_replacement;
+	t_keyvalue      *envir_str;
+	t_keyvalue      *alias;
 #ifdef HAVE_TOOLKIT
-	t_charlist    toolkit_rules;
+	t_charlist      toolkit_rules;
 #endif
 #ifdef HAVE_SSL
-    bool          require_ssl;
+    bool            require_ssl;
 #endif
 #ifdef HAVE_XSLT
-	bool          use_xslt;
+	bool            use_xslt;
 #endif
-	bool          prevent_sqli;
-	bool          prevent_xss;
-	bool          prevent_csrf;
-	bool          follow_symlinks;
-	bool          enable_path_info;
-	bool          trigger_on_cgi_status;
-	t_charlist    fast_cgi;
-	t_deny_body   *deny_body;
-
-	struct type_host *next;
-
-	bool          secure_url;
+	bool            prevent_sqli;
+	bool            prevent_xss;
+	bool            prevent_csrf;
+	bool            follow_symlinks;
+	bool            enable_path_info;
+	bool            trigger_on_cgi_status;
+	bool            secure_url;
+	t_charlist      fast_cgi;
+	t_deny_body     *deny_body;
 
 #ifdef HAVE_MONITOR
 	t_monitor_host_stats *monitor_stats;
-	bool          monitor_requests;
-	bool          monitor_host;
+	bool            monitor_requests;
+	bool            monitor_host;
 #endif
+
+	struct type_host *next;
 } t_host;
 
 typedef struct type_config {
@@ -270,6 +265,7 @@ typedef struct type_config {
 	char          *user_directory;
 	bool          user_directory_set;
 	t_iplist      *hide_proxy;
+	t_accesslist  *request_limit_mask;
 	
 	t_mimetype    *mimetype;
 	t_host        *first_host;
