@@ -250,7 +250,7 @@ int get_target_extension(t_session *session) {
  */
 int get_homedir(t_session *session, char *username) {
 	struct passwd *pwd;
-	int len;
+	size_t len;
 	char *old_root;
 
 	if (username == NULL) {
@@ -259,16 +259,15 @@ int get_homedir(t_session *session, char *username) {
 		return 404;
 	}
 
-	len = strlen(pwd->pw_dir);
 	old_root = session->host->website_root;
-	if ((session->host->website_root = (char*)malloc(len + strlen(session->config->user_directory) + 2)) == NULL) {
+
+	len = strlen(pwd->pw_dir) + strlen(session->config->user_directory) + 2;
+	if ((session->host->website_root = (char*)malloc(len)) == NULL) {
 		session->host->website_root = old_root;
 		return -1;
 	}
 
-	memcpy(session->host->website_root, pwd->pw_dir, len);
-	*(session->host->website_root + len) = '/';
-	strcpy(session->host->website_root + len + 1, session->config->user_directory);
+	sprintf(session->host->website_root, "%s/%s", pwd->pw_dir, session->config->user_directory);
 	if (register_tempdata(&(session->tempdata), session->host->website_root, tc_data) == -1) {
 		free(session->host->website_root);
 		session->host->website_root = old_root;
