@@ -392,9 +392,15 @@ int send_file(t_session *session) {
 				/* Normal file
 				 */
 #ifdef HAVE_CACHE
-				if ((cached_object = search_cache(session, session->file_on_disk)) == NULL) {
-					cached_object = add_to_cache(session, session->file_on_disk);
-				}
+#ifdef HAVE_MONITOR
+				if (session->host->monitor_host) {
+					cached_object = NULL;
+				} else
+#endif
+					if ((cached_object = search_cache(session, session->file_on_disk)) == NULL) {
+						cached_object = add_to_cache(session, session->file_on_disk);
+					}
+
 				if (cached_object != NULL) {
 					if (send_begin + send_size > cached_object->size) {
 						done_with_cached_object(cached_object, true);
