@@ -24,6 +24,7 @@
 #endif
 
 #define MAX_INPUT_SIZE KILOBYTE
+#define MAX_PATH 1024
 
 typedef struct type_line {
 	char *key, *value, *file;
@@ -550,13 +551,19 @@ int check_main_config(char *config_dir) {
 #ifdef HAVE_TOOLKIT
 void check_url_toolkit(char *config_dir, char **toolkit_id) {
 	t_line *config = NULL;
-	char input[MAX_INPUT_SIZE + 1], **id, *url;
+	char input[MAX_INPUT_SIZE + 1], **id, *url, current_dir[MAX_PATH];
 	t_url_toolkit *url_toolkit, *toolkit = NULL, *new_toolkit;
 	t_toolkit_options options;
 	bool in_rule_section = false;
 	int result = 0;
 
-	if (chdir(config_dir) == -1) {
+	if (quiet == false) {
+		printf("Using %s\n", config_dir);
+	}
+
+	if (getcwd(current_dir, MAX_PATH) == NULL) {
+		return;
+	} else if (chdir(config_dir) == -1) {
 		perror(config_dir);
 		return;
 	}
@@ -568,6 +575,11 @@ void check_url_toolkit(char *config_dir, char **toolkit_id) {
 		return;
 	}
 
+	if (chdir(current_dir) == -1) {
+		perror(current_dir);
+		return;
+	}
+
 	/* Parse the URL toolkit rules
 	 */
 	url_toolkit = NULL;
@@ -639,10 +651,7 @@ void check_url_toolkit(char *config_dir, char **toolkit_id) {
 			printf("Bad URL: missing leading slash.\n");
 		}
 
-		options.new_url = NULL;
-		options.website_root = ".";
-		options.fastcgi_server = NULL;
-		options.url_toolkit = url_toolkit;
+		init_toolkit_options(&options, ".", url_toolkit, false);
 
 		id = toolkit_id;
 		while (*id != NULL) {
