@@ -48,6 +48,7 @@ static int xml_special_chars(char *str, char **new) {
 	char *c;
 
 	if (str == NULL) {
+		*new = NULL;
 		return -1;
 	}
 	len = strlen(str);
@@ -106,9 +107,11 @@ static void add_parameter(const char **params, char *key, char *value, int *i) {
 		return;
 	}
 
-	if (forbidden_chars_present(value)) {	
+	if (forbidden_chars_present(value)) {
+		free(str);
 		return;
 	} else if ((value = strdup(value)) == NULL) {
+		free(str);
 		return;
 	}
 	url_decode(value);
@@ -119,6 +122,7 @@ static void add_parameter(const char **params, char *key, char *value, int *i) {
 		sprintf(str, "%s%c\"%s\"", key, 0, value);
 	} else {
 		free(value);
+		free(str);
 		return;
 	}
 	free(value);
@@ -643,17 +647,20 @@ int show_index(t_session *session) {
 
 		/* URL encoded
 		 */
+		link = NULL;
 		if (url_encode(file->name, &link) == -1) {
 			free(text_xml);
 			return -1;
 		} else if (add_str(&text_xml, &text_max, XML_CHUNK_LEN, &text_size, "\" url_encoded=\"") == -1) {
 			free(text_xml);
+			sfree(link);
 			return -1;
 		} else if (add_str(&text_xml, &text_max, XML_CHUNK_LEN, &text_size, link == NULL ? file->name : link) == -1) {
 			free(text_xml);
+			sfree(link);
 			return -1;
-		} else if (link != NULL) {
-			free(link);
+		} else {
+			sfree(link);
 		}
 
 		if (xml_special_chars(file->name, &link) == -1) {
@@ -661,12 +668,14 @@ int show_index(t_session *session) {
 			return -1;
 		} else if (add_str(&text_xml, &text_max, XML_CHUNK_LEN, &text_size, "\">") == -1) {
 			free(text_xml);
+			sfree(link);
 			return -1;
 		} else if (add_str(&text_xml, &text_max, XML_CHUNK_LEN, &text_size, link == NULL ? file->name : link) == -1) {
 			free(text_xml);
+			sfree(link);
 			return -1;
-		} else if (link != NULL) {
-			free(link);
+		} else {
+			sfree(link);
 		}
 
 		if (file->is_dir) {
